@@ -28,7 +28,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 '''
 # standard library
 import math
@@ -40,12 +40,12 @@ import cubicsuperpath
 import coloreffect
 import dxf_templates
 
-inkex.localize()
-
 try:
     from numpy import *
     from numpy.linalg import solve
 except:
+    # Initialize gettext for messages outside an inkex derived class
+    inkex.localize() 
     inkex.errormsg(_("Failed to import the numpy or numpy.linalg modules. These modules are required by this extension. Please install them and try again."))
     inkex.sys.exit()
 
@@ -76,7 +76,7 @@ class MyEffect(inkex.Effect):
                                      default=True)
         self.OptionParser.add_option("--units", action="store",
                                      type="string", dest="units",
-                                     default="72./90") # Points
+                                     default="72./96") # Points
         self.OptionParser.add_option("--encoding", action="store",
                                      type="string", dest="char_encode",
                                      default="latin_1")
@@ -113,14 +113,17 @@ class MyEffect(inkex.Effect):
             self.poly = [csp[0]]                            # initiallize new polyline
             self.color_LWPOLY = self.color
             self.layer_LWPOLY = self.layer
-            self.closed_LWPOLY = self.closed
         self.poly.append(csp[1])
     def LWPOLY_output(self):
         if len(self.poly) == 1:
             return
         self.handle += 1
-        self.dxf_add("  0\nLWPOLYLINE\n  5\n%x\n100\nAcDbEntity\n  8\n%s\n 62\n%d\n100\nAcDbPolyline\n 90\n%d\n 70\n%d\n" % (self.handle, self.layer_LWPOLY, self.color_LWPOLY, len(self.poly), self.closed_LWPOLY))
-        for i in range(len(self.poly)):
+        closed = 1
+        if (abs(self.poly[0][0] - self.poly[-1][0]) > .0001
+            or abs(self.poly[0][1] - self.poly[-1][1]) > .0001):
+            closed = 0
+        self.dxf_add("  0\nLWPOLYLINE\n  5\n%x\n100\nAcDbEntity\n  8\n%s\n 62\n%d\n100\nAcDbPolyline\n 90\n%d\n 70\n%d\n" % (self.handle, self.layer_LWPOLY, self.color_LWPOLY, len(self.poly) - closed, closed))
+        for i in range(len(self.poly) - closed):
             self.dxf_add(" 10\n%f\n 20\n%f\n 30\n0.0\n" % (self.poly[i][0],self.poly[i][1]))
     def dxf_spline(self,csp):
         knots = 8
@@ -195,7 +198,6 @@ class MyEffect(inkex.Effect):
                 if style['stroke'] and style['stroke'] != 'none' and style['stroke'][0:3] != 'url':
                     rgb = simplestyle.parseColor(style['stroke'])
         hsl = coloreffect.ColorEffect.rgb_to_hsl(coloreffect.ColorEffect(),rgb[0]/255.0,rgb[1]/255.0,rgb[2]/255.0)
-        self.closed = 0                                 # only for LWPOLYLINE
         self.color = 7                                  # default is black
         if hsl[2]:
             self.color = 1 + (int(6*hsl[0] + 0.5) % 6)  # use 6 hues
@@ -203,21 +205,34 @@ class MyEffect(inkex.Effect):
             d = node.get('d')
             if not d:
                 return
-            if (d[-1] == 'z' or d[-1] == 'Z'):
-                self.closed = 1
             p = cubicsuperpath.parsePath(d)
         elif node.tag == inkex.addNS('rect','svg'):
-            self.closed = 1
-            x = float(node.get('x'))
-            y = float(node.get('y'))
+            x = float(node.get('x', 0))
+            y = float(node.get('y', 0))
             width = float(node.get('width'))
             height = float(node.get('height'))
-            p = [[[x, y],[x, y],[x, y]]]
-            p.append([[x + width, y],[x + width, y],[x + width, y]])
-            p.append([[x + width, y + height],[x + width, y + height],[x + width, y + height]])
-            p.append([[x, y + height],[x, y + height],[x, y + height]])
-            p.append([[x, y],[x, y],[x, y]])
-            p = [p]
+            d = "m %s,%s %s,%s %s,%s %s,%s z" % (x, y, width, 0, 0, height, -width, 0)
+            p = cubicsuperpath.parsePath(d)
+        elif node.tag == inkex.addNS('line','svg'):
+            x1 = float(node.get('x1', 0))
+            x2 = float(node.get('x2', 0))
+            y1 = float(node.get('y1', 0))
+            y2 = float(node.get('y2', 0))
+            d = "M %s,%s L %s,%s" % (x1, y1, x2, y2)
+            p = cubicsuperpath.parsePath(d)
+        elif node.tag == inkex.addNS('circle','svg'):
+            cx = float(node.get('cx', 0))
+            cy = float(node.get('cy', 0))
+            r = float(node.get('r'))
+            d = "m %s,%s a %s,%s 0 0 1 %s,%s %s,%s 0 0 1 %s,%s z" % (cx + r, cy, r, r, -2*r, 0, r, r, 2*r, 0)
+            p = cubicsuperpath.parsePath(d)
+        elif node.tag == inkex.addNS('ellipse','svg'):
+            cx = float(node.get('cx', 0))
+            cy = float(node.get('cy', 0))
+            rx = float(node.get('rx'))
+            ry = float(node.get('ry'))
+            d = "m %s,%s a %s,%s 0 0 1 %s,%s %s,%s 0 0 1 %s,%s z" % (cx + rx, cy, rx, ry, -2*rx, 0, rx, ry, 2*rx, 0)
+            p = cubicsuperpath.parsePath(d)
         else:
             return
         trans = node.get('transform')
@@ -324,7 +339,8 @@ class MyEffect(inkex.Effect):
 
         scale = eval(self.options.units)
         if not scale:
-            scale = 25.4/90     # if no scale is specified, assume inch as baseunit
+            scale = 25.4/96     # if no scale is specified, assume inch as baseunit
+        scale /= self.unittouu('1px')
         h = self.unittouu(self.document.getroot().xpath('@height', namespaces=inkex.NSS)[0])
         self.groupmat = [[[scale, 0.0, 0.0], [0.0, -scale, h*scale]]]
         doc = self.document.getroot()
